{{
        Test FFDS1

  One instance is configured on the standard programming
  pins P30 and P31, to talk to the user.  Two more are
  then connected via a pair of free pins and configured
  to run at the max baud rate.

  The test consists of doing full-speed overlapping TX
  and RX of a large block of random data, which is then
  verified on both side.

  As the test continues to repeat, the results are sent
  back to the user via the "term" instance.

  Jonathan Dummer (lonesock) 
}}

CON

  ' NOTE!  You may need to change these to unused pins
  dummy_pin1 = 0
  dummy_pin2 = 1

  ' Clock Mode
  _clkmode        = xtal1 + pll16x
  _xinfreq        = 5_000_000
      
  COM_BAUD = 460_800 ' 9_600, 57_600, 115_200, 230_400, 250_000, 460_800
  TEST_LINK_BAUD = POSX ' clamped internally to the min allowable period

      
OBJ
  term:         "FFDS1"
  fds_test[2]:  "FFDS1"

CON
  ' stress test TX buffer (size must be < RX buffer, since it's circular)
  BUF_SIZE   = term#BUF_LEN - 1
VAR
  byte send_buf[BUF_SIZE]

PUB main : i | c1, c2, r, ok

  ' serial link back to the PC 
  term.Start( 31, 30, COM_BAUD )

  ' test linkage
  fds_test[0].Start( dummy_pin1, dummy_pin2, TEST_LINK_BAUD )
  fds_test[1].Start( dummy_pin2, dummy_pin1, TEST_LINK_BAUD )

  ' let the user hit the [Enable] button on the Parallax Serial Terminal
  waitcnt( clkfreq * 2 + cnt )

  repeat

    term.str( string( 13, "starting test", 13 ) )

    ' Do a bi-directional test
    ok := true
    ' set up a random buffer
    repeat c1 from 0 to constant( BUF_SIZE - 1 )
      ?r
      send_buf[c1] := r
    ' start with a clean slate
    fds_test[0].rxflush
    fds_test[1].rxflush
    ' send it in one go, both directions!
    fds_test[0].TxBufNoWait( @send_buf, BUF_SIZE )
    fds_test[1].TxBufNoWait( @send_buf, BUF_SIZE )
    ' wait for both cogs to finish transmitting
    fds_test[0].WaitForTX
    fds_test[1].WaitForTX
    ' read char by char
    repeat c1 from 0 to constant( BUF_SIZE - 1 )
      c2 := fds_test[0].rx
      if c2 <> send_buf[c1]
        ok := false
        term.str( string( 13, "RX 0 Failed ", 13 ) )
        term.bin( send_buf[c1], 8 )
        term.tx( " " )
        term.dec( send_buf[c1] )
        term.tx( 13 )
        term.bin( c2, 8 )
      c2 := fds_test[1].rx
      if c2 <> send_buf[c1]
        ok := false
        term.str( string( 13, "RX 1 Failed ", 13 ) )
        term.bin( send_buf[c1], 8 )
        term.tx( " " )
        term.dec( send_buf[c1] )
        term.tx( 13 )
        term.bin( c2, 8 )
    if ok
      term.str( string( 13, "Passed" ) )
    else      
      repeat
        term.str( string( 13, "The test FAILED!", 13, "Press 'C' to continue", 13 ) )        
        c1 := term.RxTime( 10000 )        
      until (c1 == "C") or (c1 == "c")

    ' test helper functions
    c1 := term.HToI( string( "2A" ) )
    term.tx( 13 )
    term.dec( c1 )
    term.tx( 13 )

    ' if the user sent a char we will echo, along with some useless info
    c1 := term.rxcheck
    term.str( string( 13, "Just testing here: " ) )
    if c1 > 0
      term.tx( c1 )
      term.tx( " " ) 
      term.bin( c1, 8 )
      term.tx( " " )
      term.hex( c1, 2 )
      term.tx( " " )
      term.dec( c1 )
      term.tx( " " )
    term.dec( i++ )

    'waitcnt( clkfreq + cnt )
      